@@ -1,6 +1,7 @@
 from __future__ import print_function
 
 import collections
+import contextlib
 import gzip
 import json
 import logging
@@ -11,10 +12,8 @@ import zlib
 from io import BytesIO
 from tornado.web import RequestHandler
 
-try:
-    from urllib.parse import urlsplit
-except ImportError:
-    from urlparse import urlsplit
+from urllib3.packages.six.moves.http_client import responses
+from urllib3.packages.six.moves.urllib.parse import urlsplit
 
 log = logging.getLogger(__name__)
 
@@ -28,8 +27,8 @@ class Response(object):
     def __call__(self, request_handler):
         status, reason = self.status.split(' ', 1)
         request_handler.set_status(int(status), reason)
-        for header,value in self.headers:
-            request_handler.add_header(header,value)
+        for header, value in self.headers:
+            request_handler.add_header(header, value)
 
         # chunked
         if isinstance(self.body, list):
@@ -48,6 +47,7 @@ class Response(object):
 
 RETRY_TEST_NAMES = collections.defaultdict(int)
 
+
 class TestingApp(RequestHandler):
     """
     Simple app that performs various operations, useful for testing an HTTP
@@ -136,8 +136,8 @@ class TestingApp(RequestHandler):
         files_ = request.files.get(param)
 
         if len(files_) != 1:
-            return Response("Expected 1 file for '%s', not %d" %(param, len(files_)),
-                                                    status='400 Bad Request')
+            return Response("Expected 1 file for '%s', not %d" % (param, len(files_)),
+                            status='400 Bad Request')
         file_ = files_[0]
 
         data = file_['body']
@@ -158,6 +158,17 @@ class TestingApp(RequestHandler):
         headers = [('Location', target)]
         return Response(status='303 See Other', headers=headers)
 
+    def multi_redirect(self, request):
+        "Performs a redirect chain based on ``redirect_codes``"
+        codes = request.params.get('redirect_codes', '200').decode('utf-8')
+        head, tail = codes.split(',', 1) if "," in codes else (codes, None)
+        status = "{0} {1}".format(head, responses[int(head)])
+        if not tail:
+            return Response("Done redirecting", status=status)
+
+        headers = [('Location', '/multi_redirect?redirect_codes=%s' % tail)]
+        return Response(status=status, headers=headers)
+
     def keepalive(self, request):
         if request.params.get('close', b'0') == b'1':
             headers = [('Connection', 'close')]
@@ -189,9 +200,8 @@ class TestingApp(RequestHandler):
         if encoding == 'gzip':
             headers = [('Content-Encoding', 'gzip')]
             file_ = BytesIO()
-            zipfile = gzip.GzipFile('', mode='w', fileobj=file_)
-            zipfile.write(data)
-            zipfile.close()
+            with contextlib.closing(gzip.GzipFile('', mode='w', fileobj=file_)) as zipfile:
+                zipfile.write(data)
             data = file_.getvalue()
         elif encoding == 'deflate':
             headers = [('Content-Encoding', 'deflate')]
@@ -245,6 +255,11 @@ class TestingApp(RequestHandler):
             data,
             headers=[('Content-Type', 'application/octet-stream')])
 
+    def status(self, request):
+        status = request.params.get("status", "200 OK")
+
+        return Response(status=status)
+
     def shutdown(self, request):
         sys.exit()
 
@@ -277,7 +292,7 @@ def _parse_header(line):
             value = p[i + 1:].strip()
             params.append((name, value))
     params = email.utils.decode_params(params)
-    params.pop(0) # get rid of the dummy again
+    params.pop(0)  # get rid of the dummy again
     pdict = {}
     for name, value in params:
         value = email.utils.collapse_rfc2231_value(value)
